/*
 ==============================================================================================================================================
 Author      : CHEONG SE HANG (dit.dhc@lostcity-studio.com)
 Description : Extension to socket library.
 ==============================================================================================================================================
 */
#include "Log.h"
#include "Socket.h"
#include <errno.h>
#include <linux/netdevice.h>
#include <netdb.h>
#include <stdio.h>
#include <string.h>
#include <sys/ioctl.h>
#include <unistd.h>
/*---------------------------------------------------------------------------------------------------------------------------------------------
 *
 *                                                              Private Variables
 *
 *-------------------------------------------------------------------------------------------------------------------------------------------*/
#ifdef LOGGING_ENABLE
static char log[ 128 ];                 /* Buffer for logging. */
#endif




/*---------------------------------------------------------------------------------------------------------------------------------------------
 *
 *                                                              Public Functions
 *
 *-------------------------------------------------------------------------------------------------------------------------------------------*/
int Accept ( int sockfd, struct sockaddr* addr, socklen_t* addrlen, const char* filename, int line ) {
    int ret;
    if ( ( ret = accept( sockfd, addr, addrlen ) ) == -1 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> accept() failed: %s", filename, line, strerror( errno ) );
        Log( log );
#endif
    }
    return ret;
}


int Bind ( int sockfd, const struct sockaddr* my_addr, socklen_t addrlen, const char* filename, int line ) {
    int ret;
    if ( ( ret = bind( sockfd, my_addr, addrlen ) ) == -1 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> bind() failed: %s", filename, line, strerror( errno ) );
        Log( log );
#endif
    }
    return ret;
}


int Close ( int fd, const char* filename, int line ) {
    int ret;
    if ( ( ret = close( fd ) ) == -1 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> close() failed: %s", filename, line, strerror( errno ) );
        Log( log );
#endif
    }
    return ret;
}


int Connect ( int sockfd, const struct sockaddr* serv_addr, socklen_t addrlen, const char* filename, int line ) {
    int ret;
    if ( ( ret = connect( sockfd, serv_addr, addrlen ) ) == -1 ) {
//#ifdef LOGGING_ENABLE
//        sprintf( log, "%s(%d) -> connect() failed: %s", filename, line, strerror( errno ) );
//        Log( log );
//#endif
    }
    return ret;
}


int IP4th () {
    int s;
    struct ifconf ifconf;
    struct ifreq ifr[ 50 ];
    if ( ( s = Socket( AF_INET, SOCK_STREAM, 0, __FILE__, __LINE__ ) ) >= 0 ) {
        ifconf.ifc_buf = ( char* )ifr;
        ifconf.ifc_len = sizeof( ifr );
        if ( ioctl( s, SIOCGIFCONF, &ifconf ) != -1 ) {
            Close( s, __FILE__, __LINE__ );
            return ( ( ( struct sockaddr_in* )&ifr[ 1 ].ifr_addr )->sin_addr.s_addr & 0xFF000000 ) >> 24;
        }
    }
    return -1;
}


int Listen ( int sockfd, int backlog, const char* filename, int line ) {
    int ret;
    if ( ( ret = listen( sockfd, backlog ) ) == -1 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> listen() failed: %s", filename, line, strerror( errno ) );
        Log( log );
#endif
    }
    return ret;
}


ssize_t Read ( int fd, void *ptr, size_t nbytes, const char* filename, int line ) {
    ssize_t n;
    if ( ( n = read( fd, ptr, nbytes ) ) == -1 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> read() error: %s", filename, line, strerror( errno ) );
        Log( log );
#endif
    }
    return n;
}


//ssize_t Sendto ( int socket, const void* message, size_t length, int flags, const struct sockaddr* dest_addr, socklen_t dest_len,
//    const char* filename, int line ) {
//    int ret;
//    if ( ( ret = sendto( socket, message, length, flags, dest_addr, dest_len ) ) == -1 ) {
//        char buf[ 128 ];
//        sprintf( buf, "%s(%d) -> sendto() failed: %s", filename, line, strerror( errno ) );
//        Log( buf );
//        return -1;
//    }
//    return ret;
//}


int Select ( int nfds, fd_set* readfds, fd_set* writefds, fd_set* exceptfds, struct timeval* timeout, const char* filename, int line ) {
    int ret;
    if ( ( ret = select( nfds, readfds, writefds, exceptfds, timeout ) ) == -1 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> select() failed: %s", filename, line, strerror( errno ) );
        Log( log );
#endif
    }
    return ret;
}


int Setsockopt ( int s, int level, int optname, const void* optval, socklen_t optlen, const char* filename, int line ) {
    int ret;
    if ( ( ret = setsockopt( s, level, optname, optval, optlen ) ) == -1 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> setsockopt() failed: %s", filename, line, strerror( errno ) );
        Log( log );
#endif
    }
    return ret;
}


int Socket ( int domain, int type, int protocol, const char* filename, int line ) {
    int ret;
    if ( ( ret = socket( domain, type, protocol ) ) == -1 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> socket() failed: %s", filename, line, strerror( errno ) );
        Log( log );
#endif
    }
    return ret;
}


//ssize_t Recvfrom ( int socket, void* buffer, size_t length, int flags, struct sockaddr* address, socklen_t* address_len,
//    const char* filename, int line ) {
//    int ret;
//    if ( ( ret = recvfrom( socket, buffer, length, flags, address, address_len ) ) == -1 ) {
//        char buf[ 128 ];
//        sprintf( buf, "%s(%d) -> recvfrom() failed: %s", filename, line, strerror( errno ) );
//        Log( buf );
//        return -1;
//    }
//    return ret;
//}


static ssize_t writen ( int fd, const void *vptr, size_t n ) {
    ssize_t    nwritten;
    size_t     nleft = n;
    const char *ptr  = vptr;
    while ( nleft > 0 ) {
        if ( ( nwritten = write( fd, ptr, nleft ) ) <= 0 ) {
            if ( nwritten < 0 && errno == EINTR )
                nwritten = 0;       /* and call write() again */
            else
                return -1;         /* error */
        }
        nleft -= nwritten;
        ptr   += nwritten;
    }
    return n;
}
void Writen ( int fd, void *ptr, size_t nbytes, const char* filename, int line ) {
    if ( writen( fd, ptr, nbytes ) != nbytes ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> writen() error: %s", filename, line, strerror( errno ) );
        Log( log );
#endif
    }
}
